#include <stdlib.h>

#include "Dizhen.h"

int main(void) {
	GLAppContext *context;
	DiDocument *doc;
	
	GLAppInit();
	context = GLAppNewContext(640, 480);
	GLAppSetContext(context);
	
	doc = DiNewDocument(0, 0, 640, 480);
	GLAppParent(doc->panel, context->mainPanel);
	
	GLAppStart();
	
	return 0;
}

DiDocument *DiNewDocument(GLint x, GLint y, GLint w, GLint h) {
	DiDocument *doc;
	
	GLAppGetMem(doc, sizeof(DiDocument));
	
	GLAppGetMem(doc->tools[kDiTool_Mesh], sizeof(DiTool) * kDiNumTools);
	doc->tools[kDiTool_Mesh]->button = GLAppNewButton(5, 5, 20, 20, "Mesh", NULL);
	doc->tools[kDiTool_Mesh]->button->flags &= kGLAppFlag_ButtonSticky;
	doc->tools[kDiTool_Mesh]->sidebar = GLAppNewPanel(10, 30, 100, h-40);
	doc->tools[kDiTool_Mesh]->Select = DiMeshToolSelect;
	doc->tools[kDiTool_Mesh]->Deselect = DiMeshToolDeselect;
	GLAppParent(doc->tools[kDiTool_Mesh]->button, doc->panel);
	
	GLAppGetMem(doc->tools[kDiTool_Tile], sizeof(DiTool) * kDiNumTools);
	doc->tools[kDiTool_Tile]->button = GLAppNewButton(30, 5, 20, 20, "Tile", NULL);
	doc->tools[kDiTool_Tile]->button->flags &= kGLAppFlag_ButtonSticky;
	doc->tools[kDiTool_Tile]->sidebar = GLAppNewPanel(10, 30, 100, h-40);
	doc->tools[kDiTool_Tile]->Select = DiTileToolSelect;
	doc->tools[kDiTool_Tile]->Deselect = DiTileToolDeselect;
	GLAppParent(doc->tools[kDiTool_Tile]->button, doc->panel);
	
	GLAppGetMem(doc->tools[kDiTool_Point], sizeof(DiTool) * kDiNumTools);
	doc->tools[kDiTool_Point]->button = GLAppNewButton(30, 5, 20, 20, "Point", NULL);
	doc->tools[kDiTool_Point]->button->flags &= kGLAppFlag_ButtonSticky;
	doc->tools[kDiTool_Point]->sidebar = GLAppNewPanel(10, 55, 100, h-40);
	doc->tools[kDiTool_Point]->Select = DiPointToolSelect;
	doc->tools[kDiTool_Point]->Deselect = DiPointToolDeselect;
	GLAppParent(doc->tools[kDiTool_Point]->button, doc->panel);
	
	GLAppGetMem(doc->world, sizeof(DynWorld));
	GLAppGetMem(doc->world->tile, sizeof(DynTile));
	doc->world->tile->size = 10;
	doc->world->tile->color[0] = 1.0;
	doc->world->tile->color[0] = 0.0;
	doc->world->tile->color[0] = 0.0;
	doc->world->tile->h1 = 0.0;
	doc->world->tile->h2 = 0.0;
	doc->world->tile->h3 = 0.0;
	doc->world->tile->h4 = 0.0;
	doc->world->tile->t1 = NULL;
	doc->world->tile->t2 = NULL;
	doc->world->tile->t3 = NULL;
	doc->world->tile->t4 = NULL;
	doc->world->tile->world = doc->world;
	DynCalcNormal(doc->world->tile);
	
	doc->panel = GLAppNewPanel(x, y, w, h);
	doc->panel->data = doc;
	doc->panel->ParseEvent = DiDocumentParseEvent;
	doc->panel->flags = 0;
	doc->view = DynNewView(120, 30, w - 130, h - 40, doc->world, 0.0, 0.0, 0.0, -45.0, 10.0);
	GLAppParent(doc->view, doc->panel);
	
	doc->tool = -1;
}

void DiDocumentParseEvent(GLAppPanel *panel, GLAppPanel *gen, GLuint type, GLuint key) {
	DiDocument *doc = (DiDocument *)panel->data;
	GLint i;
	switch (type) {
		case kGLAppEvent_ButtonPress:
			for (i = 0; i < kDiNumTools; i++) {
				if (doc->tools[i]->button == gen) {
					if (doc->tool != i) {
						if (doc->tool != kDiTool_None) {
							GLAppSendEvent(doc->tools[doc->tool]->button, doc->tools[doc->tool]->button, kGLAppEvent_ButtonRelease, 0);
						}
						doc->tool = i;
						doc->tools[i]->Select(doc);
						GLAppParent(doc->tools[i]->sidebar, doc->panel);
					}
					break;
				}
			}
			break;
		case kGLAppEvent_ButtonRelease:
			if (doc->tool >= 0) {
				if (doc->tools[doc->tool]->button == gen) {
					GLAppParent(doc->tools[doc->tool]->sidebar, NULL);
					doc->tools[doc->tool]->Deselect(doc);
					doc->tool = -1;
				}
			}
			break;
	}
}

void DiMeshToolSelect(DiDocument *doc) {
}

void DiMeshToolDeselect(DiDocument *doc) {
}

void DiTileToolSelect(DiDocument *doc) {
}

void DiTileToolDeselect(DiDocument *doc) {
}

void DiPointToolSelect(DiDocument *doc) {
}

void DiPointToolDeselect(DiDocument *doc) {
}
