#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include "DynastyView.h"

DynWorld *DynNewWorld(GLfloat lightx, GLfloat lighty, GLfloat lightz) {
	DynWorld *world;
	GLfloat length = sqrt(lightx*lightx + lighty*lighty + lightz*lightz);
	GLAppGetMem(world, sizeof(DynWorld));
	
	world->meshes = NULL;
	world->light[0] = lightx / length;
	world->light[1] = lighty / length;
	world->light[2] = lightz / length;
	
	return world;
}

DynMesh *DynAddMesh(DynWorld *world, GLfloat x, GLfloat y, GLfloat z, GLfloat xsize, GLfloat ysize,
		GLint xres, GLint yres) {
	DynMesh *mesh;
	GLint i;
	GLboolean altdivide = GL_FALSE;
	GLAppGetMem(mesh, sizeof(DynMesh));
	
	mesh->x = x;
	mesh->y = y;
	mesh->z = z;
	mesh->xsize = xsize;
	mesh->ysize = ysize;
	mesh->xres = xres;
	mesh->yres = yres;
	mesh->updated = GL_TRUE;
	mesh->mainlist = glGenLists(1);
	mesh->wirelist = glGenLists(1);
	mesh->pointlist = glGenLists(1);
	mesh->next = NULL;
	
	GLAppGetMem(mesh->heights, (xres+1)*(yres+1)*sizeof(GLfloat));
	for (i = 0; i < (xres+1)*(yres+1); i++) {
		mesh->heights[i] = 0.0;
	}
	
	mesh->heights[0] = 1.0;
	
	GLAppGetMem(mesh->tiles, xres*yres*sizeof(DynTile));
	for (i = 0; i < xres*yres; i++) {
		mesh->tiles[i].norm1[0] = 0.0;
		mesh->tiles[i].norm1[1] = 1.0;
		mesh->tiles[i].norm1[2] = 0.0;
		mesh->tiles[i].norm2[0] = 0.0;
		mesh->tiles[i].norm2[1] = 1.0;
		mesh->tiles[i].norm2[2] = 0.0;
		mesh->tiles[i].color[0] = 1.0;
		mesh->tiles[i].color[1] = 1.0;
		mesh->tiles[i].color[2] = 1.0;
		mesh->tiles[i].y = i / yres;
		mesh->tiles[i].x = i - mesh->tiles[i].y*yres;
		mesh->tiles[i].mesh = mesh;
		mesh->tiles[i].altdivide = altdivide;
		altdivide = !altdivide;
//		DynCalcNormal(mesh->tiles + i);
	}
	
	if (world->meshes) {
		DynMesh *cur = world->meshes;
		
		while (cur->next) cur = cur->next;
		
		cur->next = mesh;
	} else {
		world->meshes = mesh;
	}
	
	return mesh;
}

GLAppPanel *DynNewView(GLint x, GLint y, GLint w, GLint h, DynWorld *world, GLfloat xpos, GLfloat zpos,
		GLfloat yaw, GLfloat pitch, GLfloat zoom, GLboolean wire, GLboolean points) {
	GLAppPanel *panel;
	DynViewData *data;
	GLAppGetMem(data, sizeof(DynViewData));
	
	panel = GLAppNewPanel(x, y, w, h);
	panel->data = data;
	panel->flags = kGLAppFlag_IsSelectable;
	panel->RedrawContents = DynViewRedrawContents;
	
	data->world = world;
	data->viewx = xpos;
	data->viewz = zpos;
	data->yaw = yaw;
	data->pitch = pitch;
	data->zoom = zoom;
	data->wire = wire;
	data->points = points;
	data->projectionList = glGenLists(1);
	data->modelviewList = glGenLists(1);
	
	return panel;
}

void DynViewRedrawContents(GLAppPanel *panel) {
	DynViewData *data = (DynViewData *)panel->data;
	DynMesh *curMesh = data->world->meshes;
	GLfloat faceX = GLAppSin[(GLint)data->yaw] * GLAppCos[(GLint)data->pitch];
	GLfloat faceY = GLAppSin[(GLint)data->pitch];
	GLfloat faceZ = GLAppCos[(GLint)data->yaw] * GLAppCos[(GLint)data->pitch];
	char str[200];
	
	glGetIntegerv(GL_VIEWPORT, data->viewport);
	
	glColor3f(0.0, 0.0, 0.0);
	glBegin(GL_POLYGON);
	glVertex2i(0, 0);
	glVertex2i(panel->w, 0);
	glVertex2i(panel->w, panel->h);
	glVertex2i(0, panel->h);
	glEnd();
	
	glClear(GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glClear(GL_DEPTH_BUFFER_BIT);
	
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glNewList(data->projectionList, GL_COMPILE_AND_EXECUTE);
	gluPerspective(55.0, (GLfloat)panel->w / (GLfloat)panel->h, 1.0, 1000.0);
	glEndList();
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	glNewList(data->modelviewList, GL_COMPILE_AND_EXECUTE);
	gluLookAt(data->viewx - data->zoom * faceX, 0.0 - data->zoom * faceY, data->viewz - data->zoom * faceZ,
		data->viewx, 0.0, data->viewz, 0.0, 1.0, 0.0);
	glEndList();
	
	while (curMesh) {
		if (curMesh->updated) {
			GLint i;
			GLfloat x = curMesh->x;
			GLfloat y = curMesh->z;
			GLfloat xsize = curMesh->xsize / curMesh->xres;
			GLfloat ysize = curMesh->ysize / curMesh->yres;
			GLfloat size = curMesh->xres * curMesh->yres;
			
			glNewList(curMesh->mainlist, GL_COMPILE);
			for (i = 0; i < size; i++) {
				DynTile *tile = curMesh->tiles + i;
				GLfloat dot;
				GLfloat minx = x;
				GLfloat maxx = x + xsize;
				GLfloat miny = y;
				GLfloat maxy = y + ysize;
				GLint vertex = i / curMesh->xres + i;
				GLfloat h1 = curMesh->heights[vertex];
				GLfloat h2 = curMesh->heights[vertex + 1];
				GLfloat h3 = curMesh->heights[vertex + curMesh->xres + 1];
				GLfloat h4 = curMesh->heights[vertex + curMesh->xres + 2];
				
				glLoadName(i);
				dot = -tile->norm1[0] * data->world->light[0] - tile->norm1[1] * data->world->light[1]
					- tile->norm1[2] * data->world->light[2];
				dot = GLAppClamp(dot, 0.2, 1.0);
				glColor3f(tile->color[0] * dot, tile->color[1] * dot, tile->color[2] * dot);
				glBegin(GL_POLYGON);
				if (tile->altdivide) {
					glVertex3f(minx, h1, miny);
					glVertex3f(maxx, h2, miny);
					glVertex3f(minx, h3, maxy);
				} else {
					glVertex3f(minx, h1, miny);
					glVertex3f(maxx, h4, maxy);
					glVertex3f(minx, h3, maxy);
				}
				glEnd();
				
				dot = -tile->norm2[0] * data->world->light[0] - tile->norm2[1] * data->world->light[1]
					- tile->norm2[2] * data->world->light[2];
				dot = GLAppClamp(dot, 0.2, 1.0);
				glColor3f(tile->color[0] * dot, tile->color[1] * dot, tile->color[2] * dot);
				glBegin(GL_POLYGON);
				if (tile->altdivide) {
					glVertex3f(maxx, h2, miny);
					glVertex3f(maxx, h4, maxy);
					glVertex3f(minx, h3, maxy);
				} else {
					glVertex3f(minx, h1, miny);
					glVertex3f(maxx, h2, miny);
					glVertex3f(maxx, h4, maxy);
				}
				glEnd();
				
				x += xsize;
				if (x >= (curMesh->x + curMesh->xsize)) {
					x = curMesh->x;
					y += ysize;
				}
			}
			glEndList();
			
			x = curMesh->x;
			y = curMesh->z;
			glNewList(curMesh->wirelist, GL_COMPILE);
			for (i = 0; i < size; i++) {
				DynTile *tile = curMesh->tiles + i;
				GLfloat minx = x;
				GLfloat maxx = x + xsize;
				GLfloat miny = y;
				GLfloat maxy = y + ysize;
				GLint vertex = i / curMesh->xres + i;
				GLfloat h1 = curMesh->heights[vertex];
				GLfloat h2 = curMesh->heights[vertex + 1];
				GLfloat h3 = curMesh->heights[vertex + curMesh->xres + 1];
				GLfloat h4 = curMesh->heights[vertex + curMesh->xres + 2];
				
				glLoadName(i);
				glColor3f(1.0, 1.0, 1.0);
				glBegin(GL_LINE_STRIP);
				if (tile->altdivide) {
					glVertex3f(maxx, h2, miny);
					glVertex3f(minx, h1, miny);
					glVertex3f(minx, h3, maxy);
					glVertex3f(maxx, h2, miny);
					glVertex3f(maxx, h4, maxy);
					glVertex3f(minx, h3, maxy);
				} else {
					glVertex3f(minx, h1, miny);
					glVertex3f(maxx, h2, miny);
					glVertex3f(maxx, h4, maxy);
					glVertex3f(minx, h1, miny);
					glVertex3f(minx, h3, maxy);
					glVertex3f(maxx, h4, maxy);
				}
				glEnd();
				
				x += xsize;
				if (x >= (curMesh->x + curMesh->xsize)) {
					x = curMesh->x;
					y += ysize;
				}
			}
			glEndList();
			
			x = curMesh->x;
			y = curMesh->z;
			size = (curMesh->xres+1) * (curMesh->yres+1);
			glNewList(curMesh->pointlist, GL_COMPILE);
			glPointSize(3.0);
			glColor3f(1.0, 1.0, 1.0);
			for (i = 0; i < size; i++) {
				glLoadName(i);
				glBegin(GL_POINTS);
				glVertex3f(x, curMesh->heights[i], y);
				glEnd();
				
				x += xsize;
				if (x >= (curMesh->x + curMesh->xsize + xsize)) {
					x = curMesh->x;
					y += ysize;
				}
			}
			glEndList();
		}
		
		glCallList(curMesh->mainlist);
		if (data->wire) glCallList(curMesh->wirelist);
		if (data->points) glCallList(curMesh->pointlist);
		
		curMesh = curMesh->next;
	}
	
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	
	glDisable(GL_DEPTH_TEST);
	
	glColor3f(1.0, 1.0, 1.0);
	sprintf(str, "Yaw: %.2f, Pitch: %.2f", data->yaw, data->pitch);
	GLAppDrawString(10, 10, 10, str);
	sprintf(str, "Facing: <%.5f, %.5f, %.5f>", faceX, faceY, faceZ);
	GLAppDrawString(10, 20, 10, str);
}

void DynCalcNormal(DynTile *tile) {
	GLint tilepos = tile->x + tile->y*tile->mesh->xres;
	GLfloat xsize = tile->mesh->xsize / tile->mesh->xres;
	GLfloat ysize = tile->mesh->ysize / tile->mesh->yres;
	GLint vertex = tilepos / tile->mesh->xres + tilepos;
	GLfloat h1 = tile->mesh->heights[vertex];
	GLfloat h2 = tile->mesh->heights[vertex + 1];
	GLfloat h3 = tile->mesh->heights[vertex + tile->mesh->xres + 1];
	GLfloat h4 = tile->mesh->heights[vertex + tile->mesh->xres + 2];
	GLfloat ux, uy, uz, vx, vy, vz, length;
	
	if (tile->altdivide) {
		ux = xsize;
		uy = h2 - h1;
		uz = 0;
	} else {
		ux = 0;
		uy = h1 - h3;
		uz = ysize;
	}
	length = sqrt(ux*ux + uy*uy + uz*uz);
	ux /= length;
	uy /= length;
	uz /= length;
	
	if (tile->altdivide) {
		vx = 0;
		vy = h3 - h1;
		vz = -ysize;
	} else {
		vx = xsize;
		vy = h4 - h3;
		vz = 0;
	}
	length = sqrt(vx*vx + vy*vy + vz*vz);
	vx /= length;
	vy /= length;
	vz /= length;
	
	tile->norm1[0] = uy*vz - uz*vy;
	tile->norm1[1] = uz*vx - ux*vz;
	tile->norm1[2] = ux*vy - uy*vx;
	
	if (tile->altdivide) {
		ux = -xsize;
		uy = h3 - h4;
		uz = 0;
	} else {
		ux = 0;
		uy = h4 - h2;
		uz = -ysize;
	}
	length = sqrt(ux*ux + uy*uy + uz*uz);
	ux /= length;
	uy /= length;
	uz /= length;
	
	if (tile->altdivide) {
		vx = 0;
		vy = h2 - h4;
		vz = ysize;
	} else {
		vx = -xsize;
		vy = h1 - h2;
		vz = 0;
	}
	length = sqrt(vx*vx + vy*vy + vz*vz);
	vx /= length;
	vy /= length;
	vz /= length;
	
	tile->norm2[0] = uy*vz - uz*vy;
	tile->norm2[1] = uz*vx - ux*vz;
	tile->norm2[2] = ux*vy - uy*vx;
}
